package cz.finance.blocking.analytics.controller;

import cz.finance.blocking.analytics.client.TransactionStorageClient;
import cz.finance.blocking.analytics.client.UserManagementClient;
import cz.finance.blocking.analytics.model.Transaction;
import cz.finance.blocking.analytics.model.User;
import lombok.RequiredArgsConstructor;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.stream.Collectors;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/analytics")
@RequiredArgsConstructor
public class AnalyticsController {

    private final TransactionStorageClient transactionStorageClient;
    private final UserManagementClient userManagementClient;

    @GetMapping("/spending")
    public BigDecimal getAverageSpending(@RequestParam("fromAge") Integer fromAge, @RequestParam("toAge") Integer toAge) {
        var usernames = userManagementClient.getAllByAgeRange(fromAge, toAge).stream()
                .map(User::getUsername)
                .collect(Collectors.toList());
        var transactions = transactionStorageClient.getAllByUsernames(usernames);
        return transactions.stream()
                .map(Transaction::getAmount)
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .divide(BigDecimal.valueOf(transactions.size()), RoundingMode.DOWN);
    }
}
