package cz.finance.blocking.usermanagement.controller;

import cz.finance.blocking.usermanagement.entity.User;
import cz.finance.blocking.usermanagement.properties.SecurityProperties;
import cz.finance.blocking.usermanagement.repository.UserRepository;
import io.jsonwebtoken.JwtException;
import io.jsonwebtoken.Jwts;
import lombok.RequiredArgsConstructor;

import java.util.List;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/users")
@RequiredArgsConstructor
public class UserController {

    private final UserRepository userRepository;
    private final SecurityProperties securityProperties;

    @GetMapping
    public List<User> getAllUsers(@RequestParam(value = "fromAge", required = false) Integer fromAge,
            @RequestParam(value = "toAge", required = false) Integer toAge) {
        return fromAge != null ? userRepository.findByAgeRange(fromAge, toAge) : userRepository.findAll();
    }

    @GetMapping("/{username}")
    public User getUser(@PathVariable String username) {
        return userRepository.findByUsername(username);
    }

    @PostMapping
    public User createUser(@RequestBody User userData) {
        return userRepository.save(userData);
    }

    @DeleteMapping("/{username}")
    public void deleteUser(@PathVariable String username) {
        userRepository.deleteByUsername(username);
    }

    @GetMapping("/hash")
    public int calculateHash(User userData) {
        return userData.hashCode();
    }

    @GetMapping("/verification")
    public ResponseEntity verifyToken(@RequestParam("token") String jwtToken) {
        try {
            Jwts.parser().setSigningKey(securityProperties.getBase64Key()).parseClaimsJws(jwtToken);
            return ResponseEntity.noContent().build();
        } catch (JwtException e) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).build();
        }
    }
}
