package cz.finance.reactive.analytics.client.config;

import com.netflix.client.DefaultLoadBalancerRetryHandler;
import com.netflix.loadbalancer.reactive.LoadBalancerCommand;
import cz.finance.reactive.analytics.client.TransactionStorageClient;
import cz.finance.reactive.analytics.client.UserManagementClient;
import reactivefeign.cloud.CloudReactiveFeign;
import reactivefeign.webclient.WebReactiveFeign;

import org.springframework.cloud.netflix.ribbon.RibbonClient;
import org.springframework.cloud.netflix.ribbon.RibbonClients;
import org.springframework.cloud.netflix.ribbon.SpringClientFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
@RibbonClients({
        @RibbonClient(name = FeignConfig.TRANSACTION_STORAGE_NAME)
})
public class FeignConfig {

    static final String TRANSACTION_STORAGE_NAME = "transaction-storage-service-reactive";
    static final String USER_MANAGEMENT_SERVICE = "user-management-service-reactive";

    @Bean
    public TransactionStorageClient transactionStorageClient(SpringClientFactory clientFactory) {
        return CloudReactiveFeign.<TransactionStorageClient>builder(WebReactiveFeign.builder())
                .disableHystrix()
                .setLoadBalancerCommandFactory(s -> LoadBalancerCommand.builder()
                        .withLoadBalancer(clientFactory.getLoadBalancer(s))
                        .withRetryHandler(new DefaultLoadBalancerRetryHandler(1, 1, true))
                        .build())
                .target(TransactionStorageClient.class, "http://" + TRANSACTION_STORAGE_NAME);
    }

    @Bean
    public UserManagementClient userManagementClient(SpringClientFactory clientFactory) {
        return CloudReactiveFeign.<UserManagementClient>builder(WebReactiveFeign.builder())
                .disableHystrix()
                .setLoadBalancerCommandFactory(s -> LoadBalancerCommand.builder()
                        .withLoadBalancer(clientFactory.getLoadBalancer(s))
                        .withRetryHandler(new DefaultLoadBalancerRetryHandler(1, 1, true))
                        .build())
                .target(UserManagementClient.class, "http://" + USER_MANAGEMENT_SERVICE);
    }

}
