package cz.finance.reactive.analytics.controller;

import cz.finance.reactive.analytics.client.TransactionStorageClient;
import cz.finance.reactive.analytics.client.TransactionStorageClient.Username;
import cz.finance.reactive.analytics.client.UserManagementClient;
import cz.finance.reactive.analytics.model.Transaction;
import lombok.RequiredArgsConstructor;
import reactor.core.publisher.Mono;
import reactor.util.function.Tuples;

import java.math.BigDecimal;
import java.math.RoundingMode;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RequiredArgsConstructor
@RestController
@RequestMapping("analytics")
public class AnalyticsController {

    private final TransactionStorageClient transactionStorageClient;
    private final UserManagementClient userManagementClient;

    @GetMapping("/spending")
    public Mono<BigDecimal> getAverageSpending(@RequestParam("fromAge") Integer fromAge, @RequestParam("toAge") Integer toAge) {
        return userManagementClient.getAllByAgeRange(fromAge, toAge)
                .map(user -> new Username(user.getUsername()))
                .transform(transactionStorageClient::getAllByUsernames)
                .map(Transaction::getAmount)
                .reduce(Tuples.of(BigDecimal.ZERO, 0), (t, bigDecimal) -> Tuples.of(t.getT1().add(bigDecimal), t.getT2() + 1))
                .map(t -> t.getT1().divide(BigDecimal.valueOf(t.getT2()), RoundingMode.DOWN));
    }

}
