package cz.finance.reactive.transactionstorage.controller;

import cz.finance.reactive.transactionstorage.entity.Transaction;
import cz.finance.reactive.transactionstorage.repository.TransactionRepository;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import org.springframework.data.domain.PageRequest;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/transactions")
@RequiredArgsConstructor
public class TransactionController {

    private final TransactionRepository transactionRepository;

    @GetMapping
    public Flux<Transaction> getAllTransactions(@RequestParam(value = "limit", required = false) Integer limit) {
        if (limit != null) {
            return transactionRepository.findAll(PageRequest.of(0, limit));
        }
        return transactionRepository.findAll();
    }

    @PutMapping("/batch")
    public Flux<Transaction> getAllTransactionsBatch(@RequestBody Flux<Username> usernames) {
        return usernames
                .map(Username::getUsername)
                .buffer()
                .flatMap(transactionRepository::findAllByUsernames);
    }

    @PostMapping
    public Mono<Void> createTransaction(@RequestBody Mono<Transaction> transaction) {
        return transaction.flatMap(transactionRepository::save).then();
    }

    @Getter
    @RequiredArgsConstructor
    static class Username {
        private final String username;
    }
}
