package cz.finance.reactive.usermanagement.controller;

import cz.finance.reactive.usermanagement.entity.User;
import cz.finance.reactive.usermanagement.properties.SecurityProperties;
import cz.finance.reactive.usermanagement.repository.UserRepository;
import io.jsonwebtoken.JwtException;
import io.jsonwebtoken.Jwts;
import lombok.RequiredArgsConstructor;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/users")
@RequiredArgsConstructor
public class UserController {

    private final UserRepository userRepository;
    private final SecurityProperties securityProperties;

    @GetMapping
    public Flux<User> getAllUsers(@RequestParam(value = "fromAge", required = false) Integer fromAge,
            @RequestParam(value = "toAge", required = false) Integer toAge) {
        return fromAge != null ? userRepository.findByAgeRange(fromAge, toAge) : userRepository.findAll();
    }

    @GetMapping("/{username}")
    public Mono<User> getUser(@PathVariable String username) {
        return userRepository.findByUsername(username);
    }

    @PostMapping
    public Mono<User> createUser(@RequestBody Mono<User> userData) {
        return userData.flatMap(userRepository::save);
    }

    @DeleteMapping("/{username}")
    public Mono<Void> deleteUser(@PathVariable String username) {
        return userRepository.deleteByUsername(username).then();
    }

    @GetMapping("/hash")
    public Mono<Integer> calculateHash(Mono<User> userData) {
        return userData.map(User::hashCode);
    }

    @GetMapping("/verification")
    public Mono<ResponseEntity> verifyToken(@RequestParam("token") String jwtToken) {
        try {
            Jwts.parser().setSigningKey(securityProperties.getBase64Key()).parseClaimsJws(jwtToken);
            return Mono.just(ResponseEntity.noContent().build());
        } catch (JwtException e) {
            return Mono.just(ResponseEntity.status(HttpStatus.UNAUTHORIZED).build());
        }
    }
}
