resource "aws_vpc" "finance-services-vpc" {
  cidr_block = "10.0.0.0/16"
  enable_dns_hostnames = true
  enable_dns_support = true
  tags {
    Name = "Finance services VPC"
  }
}

resource "aws_subnet" "finance-subnet" {
  vpc_id = "${aws_vpc.finance-services-vpc.id}"
  cidr_block = "10.0.1.0/24"
  availability_zone = "eu-west-1a"
  tags {
    Name = "Finance services subnet"
  }
}

resource "aws_internet_gateway" "finance-gateway" {
  vpc_id = "${aws_vpc.finance-services-vpc.id}"
  tags {
    Name = "Finance services IGW"
  }
}

resource "aws_route_table" "finance-routes" {
  vpc_id = "${aws_vpc.finance-services-vpc.id}"
  route {
    cidr_block = "0.0.0.0/0"
    gateway_id = "${aws_internet_gateway.finance-gateway.id}"
  }
  tags {
    Name = "Finance services routes"
  }
}

resource "aws_route_table_association" "finance-subnet-routes" {
  subnet_id = "${aws_subnet.finance-subnet.id}"
  route_table_id = "${aws_route_table.finance-routes.id}"
}

resource "aws_security_group" "finance-public-sg" {
  name = "finance-public-sg"
  description = "Allow incoming HTTP connections, SSH access and inter-group communication"
  vpc_id = "${aws_vpc.finance-services-vpc.id}"
  ingress {
    from_port = 80
    to_port = 80
    protocol = "tcp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  ingress {
    from_port = 443
    to_port = 443
    protocol = "tcp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  ingress {
    from_port = -1
    to_port = -1
    protocol = "icmp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  ingress {
    from_port = 22
    to_port = 22
    protocol = "tcp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  ingress {
    from_port = 0
    to_port = 0
    protocol = -1
    self = true
  }
  egress {
    from_port = 80
    to_port = 80
    protocol = "tcp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  egress {
    from_port = 443
    to_port = 443
    protocol = "tcp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  egress {
    from_port = -1
    to_port = -1
    protocol = "icmp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  egress {
    from_port = 0
    to_port = 0
    protocol = -1
    self = true
  }
  egress {
    from_port = 27017
    to_port = 27017
    protocol = "tcp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  tags {
    Name = "Finance public SG"
  }
}

resource "aws_security_group" "finance-mongo-sg" {
  name = "finance-mongo-sg"
  description = "Allow incoming mongo and SSH"
  vpc_id = "${aws_vpc.finance-services-vpc.id}"
  ingress {
    from_port = 27017
    to_port = 27017
    protocol = "tcp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  ingress {
    from_port = 22
    to_port = 22
    protocol = "tcp"
    cidr_blocks = [
      "0.0.0.0/0"
    ]
  }
  tags {
    Name = "Finance Mongo SG"
  }
}