provider "aws" {
  access_key = "${var.access_key}"
  secret_key = "${var.secret_key}"
  region = "${var.region}"
}

resource "aws_instance" "discovery-service" {
  ami = "${lookup(var.amis, "docker")}"
  instance_type = "t2.micro"
  subnet_id = "${aws_subnet.finance-subnet.id}"
  vpc_security_group_ids = ["${aws_security_group.finance-public-sg.id}"]
  associate_public_ip_address = true
  tags {
    Name = "discovery-service"
  }
  provisioner "remote-exec" {
    inline = [
      "docker run -d --restart=always --network host vitjouda/thesis-finance:discovery-service"
    ]
  }
  connection {
    type = "ssh"
    user = "ubuntu"
    private_key = "${file("../keys/aws.pem")}"
  }
}

resource "aws_instance" "spring-boot-admin" {
  ami = "${lookup(var.amis, "docker")}"
  instance_type = "t2.micro"
  subnet_id = "${aws_subnet.finance-subnet.id}"
  vpc_security_group_ids = ["${aws_security_group.finance-public-sg.id}"]
  associate_public_ip_address = true
  tags {
    Name = "spring-boot-admin"
  }
  provisioner "remote-exec" {
    inline = [
      "docker run -d --restart=always --network host -e SERVER_PORT=80 -e EUREKA_CLIENT_SERVICEURL_DEFAULTZONE='http://discovery.finance-local.cloud:8761/eureka' vitjouda/thesis-finance:spring-boot-admin"
    ]
  }
  connection {
    type = "ssh"
    user = "ubuntu"
    private_key = "${file("../keys/aws.pem")}"
  }
}

resource "aws_instance" "mongo-db" {
  ami = "${lookup(var.amis, "mongo")}"
  instance_type = "t2.large"
  subnet_id = "${aws_subnet.finance-subnet.id}"
  vpc_security_group_ids = ["${aws_security_group.finance-mongo-sg.id}"]
  associate_public_ip_address = true
  tags {
    Name = "mongo-db"
  }
}

resource "aws_instance" "analytics-service" {
  ami = "${lookup(var.amis, "docker")}"
  instance_type = "t2.micro"
  subnet_id = "${aws_subnet.finance-subnet.id}"
  vpc_security_group_ids = ["${aws_security_group.finance-public-sg.id}"]
  associate_public_ip_address = true
  tags {
    Name = "analytics-service"
  }
}

resource "aws_instance" "transaction-storage-service" {
  ami = "${lookup(var.amis, "docker")}"
  instance_type = "t2.micro"
  subnet_id = "${aws_subnet.finance-subnet.id}"
  vpc_security_group_ids = ["${aws_security_group.finance-public-sg.id}"]
  associate_public_ip_address = true
  tags {
    Name = "transaction-storage-service"
  }
}

resource "aws_instance" "user-management-service" {
  ami = "${lookup(var.amis, "docker")}"
  instance_type = "t2.micro"
  subnet_id = "${aws_subnet.finance-subnet.id}"
  vpc_security_group_ids = ["${aws_security_group.finance-public-sg.id}"]
  associate_public_ip_address = true
  tags {
    Name = "user-management-service"
  }
}

resource "aws_instance" "gatling-runner" {
  ami = "${lookup(var.amis, "gatling")}"
  instance_type = "t2.large"
  subnet_id = "${aws_subnet.finance-subnet.id}"
  vpc_security_group_ids = ["${aws_security_group.finance-public-sg.id}"]
  associate_public_ip_address = true
  tags {
    Name = "gatling-runner"
  }
}