package cz.finance.generator;

import org.apache.commons.lang3.RandomStringUtils;

import javax.annotation.PostConstruct;

import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

@Component
public class DataGenerator {

    private static final int AMOUNT_RANGE = 1_000_000;
    private static final int USERNAME_LENGTH = 10;

    @Autowired
    @Qualifier("transactionTemplate")
    private MongoTemplate transactionTemplate;

    @Autowired
    @Qualifier("userTemplate")
    private MongoTemplate userTemplate;

    @PostConstruct
    public void generateData() {
        generateUsersWithTransaction(10, 100, 20, 29);
        generateUsersWithTransaction(50, 300, 30, 39);
        generateUsersWithTransaction(200, 600, 40, 49);
    }

    private void generateUsersWithTransaction(int usersCount, int transactionsCount, int from, int to) {
        var users = usersInRange(usersCount, from, to);
        var transactions = transaction(transactionsCount, users);
        users.forEach(user -> userTemplate.save(user));
        transactions.forEach(transaction -> transactionTemplate.save(transaction));
    }

    private List<User> usersInRange(int count, int from, int to) {
        return IntStream.range(0, count).mapToObj(value ->
                User.builder()
                        .username(RandomStringUtils.randomAlphabetic(USERNAME_LENGTH))
                        .name("Test")
                        .surname("User")
                        .age(randomInRange(from, to))
                        .build())
                .collect(Collectors.toList());
    }

    private List<Transaction> transaction(int count, List<User> users) {
        return IntStream.range(0, count).mapToObj(value ->
                Transaction.builder()
                        .amount(new BigDecimal(randomInRange(-AMOUNT_RANGE, AMOUNT_RANGE)))
                        .username(users.get(randomInRange(0, users.size() - 1)).getUsername())
                        .build())
                .collect(Collectors.toList());
    }

    private int randomInRange(int from, int to) {
        return (int) (Math.random() * ((to - from) + 1)) + from;
    }

}
