package thesis

import io.gatling.core.Predef._
import io.gatling.http.Predef._
import scala.concurrent.duration._
import scala.util.Random

class CreateGetUserSimulation extends Simulation {

  val USER_MANAGEMENT_URL = "http://localhost:8082"

  val httpProtocol = http
    .baseUrl(USER_MANAGEMENT_URL)
    .acceptHeader("application/json")

  val usernameFeeder = Iterator.continually(
    Map("randUsername" ->
      (Random.alphanumeric.take(11).mkString)
    )
  )

  val scn = scenario("CreateGetUserScenario")
    .feed(usernameFeeder)
    .exec(http("create")
      .post("/users")
      .body(
        StringBody(
          """{
            "username": "${randUsername}",
            "name": "Mr",
            "surname": "Mock"
          }
          """))
      .asJson
    )
    .exec(http("get")
      .get("/users/${randUsername}"))

  setUp(
    scn.inject(atOnceUsers(250))
  ).protocols(httpProtocol)
}